#!/usr/bin/env python3
from matplotlib import pyplot as plt
import matplotlib.ticker as mtick
import matplotlib.dates as mdates
from UliEngineering.EngineerIO import format_value

def set_axis_tick_format_func(func, axis, set_major=True, set_minor=True):
    """
    Set the tick formatter for an X or Y axis to set_axis_tick_format_func
    Typically, this is not called directly.
    Usually you should use set_xaxis_tick_format_func() or set_yaxis_tick_format_func()
    instead.
    """
    formatter = mtick.FuncFormatter(func)
    if set_major:
        axis.set_major_formatter(formatter)
    if set_minor:
        axis.set_minor_formatter(formatter)

def set_yaxis_tick_format_func(func, ax=None, set_major=True, set_minor=True):
    """
    For the current Y axis, set <func(value, index)> as the (major & minor) tick formatter.
    This is often used with lambda functions.
 
    Example usage:

    :param Function func: The formatter function (e.g. lambda value, index: ...)
    :param Axis ax: You can specify an ax object here. If None, plt.gca() is used.
    :param bool set_major: If True, set the formatter as major tick formatter
    :param bool set_minor: If True, set the formatter as minor tick formatter
    """
    if ax is None:
        ax = plt.gca()
    set_axis_tick_format_func(func, ax.yaxis, set_major=set_major, set_minor=set_minor)


def set_xaxis_tick_format_func(func, ax=None, set_major=True, set_minor=True):
    """
    For the current Y axis, set <func(value, index)> as the (major & minor) tick formatter.
    This is often used with lambda functions.
 
    Example usage:

    :param Function func: The formatter function (e.g. lambda value, index: ...)
    :param Axis ax: You can specify an ax object here. If None, plt.gca() is used.
    :param bool set_major: If True, set the formatter as major tick formatter
    :param bool set_minor: If True, set the formatter as minor tick formatter
    """
    if ax is None:
        ax = plt.gca()
    set_axis_tick_format_func(func, ax.xaxis, set_major=set_major, set_minor=set_minor)

def set_axis_tick_format_float(axis, decimals=3, unit=None, set_major=True, set_minor=True):
    """
    Set the number of decimals and the unit for formatting the given X or Y axis.
    Typically, this is not called directly.
    Usually you should use set_xaxis_tick_format_float() or set_yaxis_tick_format_float()
    instead.
    """
    # Create format string
    formatstr = f"%.{decimals}f"
    if unit is not None:
        formatstr += f" {unit}"
    # Set formatter
    formatter = mtick.FormatStrFormatter(formatstr)
    if set_major:
        axis.set_major_formatter(formatter)
    if set_minor:
        axis.set_minor_formatter(formatter)
        
def set_axis_tick_format_si_unit(axis, unit="V", significant_digits=3, set_major=True, set_minor=True):
    """
    Use UliEngineering format_value() to format the axis ticks with SI units,
    such as "mV" or "uA".
    """
    set_axis_tick_format_func(lambda value, index: format_value(value, unit=unit, significant_digits=significant_digits), axis, set_major=set_major, set_minor=set_minor)

def set_yaxis_tick_format_si_unit(unit="V", significant_digits=3, ax=None, set_major=True, set_minor=True):
    """
    Use UliEngineering format_value() to format the Y axis ticks with SI units,
    """
    # Create ax unless specfied
    if ax is None:
        ax = plt.gca()
    # Set formatter
    set_axis_tick_format_si_unit(ax.yaxis, unit=unit, significant_digits=significant_digits, set_major=set_major, set_minor=set_minor)
    
def set_xaxis_tick_format_si_unit(unit="V", significant_digits=3, ax=None, set_major=True, set_minor=True):
    """
    Use UliEngineering format_value() to format the X axis ticks with SI units,
    """
    # Create ax unless specfied
    if ax is None:
        ax = plt.gca()
    # Set formatter
    set_axis_tick_format_si_unit(ax.xaxis, unit=unit, significant_digits=significant_digits, set_major=set_major, set_minor=set_minor)

def set_yaxis_tick_format_float(decimals=3, unit=None, ax=None, set_major=True, set_minor=True):
    """
    Set the number of decimals and the unit for formatting the Y axis.
    """
    # Create ax unless specfied
    if ax is None:
        ax = plt.gca()
    # Set formatter
    set_axis_tick_format_float(ax.yaxis, decimals=decimals, unit=unit, set_major=set_major, set_minor=set_minor)

def set_xaxis_tick_format_float(decimals=3, unit=None, ax=None, set_major=True, set_minor=True):
    """
    Set the number of decimals and the unit for formatting the Y axis.
    """
    # Create ax unless specfied
    if ax is None:
        ax = plt.gca()
    # Set formatter
    set_axis_tick_format_float(ax.xaxis, decimals=decimals, unit=unit, set_major=set_major, set_minor=set_minor)

def set_axis_tick_format_datetime(axis, dateformat="%Y%m%d %H%M%S", tz=None, set_major=True, set_minor=True):
    """
    Set the date format for formatting the given X or Y axis.
    Typically, this is not called directly.
    Usually you should use set_xaxis_tick_format_datetime() or set_yaxis_tick_format_datetime()
    instead.

    :param axis: The axis (X or Y) to set the formatter for
    :param str dateformat: The strftime date format string, e.g. "%Y%m%d %H%M%S"
    :param datetime.tzinfo tz: The time zone for the ticks, uses matplotlib default if None
    :param bool set_major: If True, set the formatter as major tick formatter
    :param bool set_minor: If True, set the formatter as minor tick formatter
    """
    # Set formatter
    formatter = mdates.DateFormatter(dateformat, tz=tz)
    if set_major:
        axis.set_major_formatter(formatter)
    if set_minor:
        axis.set_minor_formatter(formatter)


def set_yaxis_tick_format_datetime(dateformat="%Y-%m-%d %H:%M:%S", tz=None, ax=None, set_major=True, set_minor=True):
    """
    Configure the Y axis with datetime format ticks
    
    :param str dateformat: The strftime date format string, e.g. "%Y%m%d %H%M%S"
    :param datetime.tzinfo tz: The time zone for the ticks, uses matplotlib default if None
    :param Axis ax: You can specify an ax object here. If None, plt.gca() is used.
    :param bool set_major: If True, set the formatter as major tick formatter
    :param bool set_minor: If True, set the formatter as minor tick formatter
    """
    # Create ax unless specfied
    if ax is None:
        ax = plt.gca()
    # Set formatter
    set_axis_tick_format_datetime(ax.yaxis, dateformat=dateformat, tz=tz, set_major=set_major, set_minor=set_minor)

def set_xaxis_tick_format_datetime(dateformat="%Y-%m-%d %H:%M:%S", tz=None, ax=None, set_major=True, set_minor=True):
    """
    Configure the X axis with datetime format ticks
    
    :param str dateformat: The strftime date format string, e.g. "%Y%m%d %H%M%S"
    :param datetime.tzinfo tz: The time zone for the ticks, uses matplotlib default if None
    :param Axis ax: You can specify an ax object here. If None, plt.gca() is used.
    :param bool set_major: If True, set the formatter as major tick formatter
    :param bool set_minor: If True, set the formatter as minor tick formatter
    """
    # Create ax unless specfied
    if ax is None:
        ax = plt.gca()
    # Set formatter
    set_axis_tick_format_datetime(ax.xaxis, dateformat=dateformat, tz=tz, set_major=set_major, set_minor=set_minor)

def set_axis_tick_format_percent(axis, fullscale=1.0, set_major=True, set_minor=True):
    """
    Set the percent format for formatting the given X or Y axis.
    Typically, this is not called directly.
    Usually you should use set_xaxis_tick_format_percent() or set_yaxis_tick_format_percent()
    instead.

    :param axis: The axis (X or Y) to set the formatter for
    :param fullscale: What value is considered to be 100%. Use 100.0 for values that are already scaled as percent.
    :param bool set_major: If True, set the formatter as major tick formatter
    :param bool set_minor: If True, set the formatter as minor tick formatter
    """
    # Set formatter
    formatter = mtick.PercentFormatter(fullscale)
    if set_major:
        axis.set_major_formatter(formatter)
    if set_minor:
        axis.set_minor_formatter(formatter)


def set_yaxis_tick_format_percent(fullscale=1.0, ax=None, set_major=True, set_minor=True):
    """
    Set the number of decimals and the unit for formatting the Y axis.
    
    :param fullscale: What value is considered to be 100%. Use 100.0 for values that are already scaled as percent.
    :param Axis ax: You can specify an ax object here. If None, plt.gca() is used.
    :param bool set_major: If True, set the formatter as major tick formatter
    :param bool set_minor: If True, set the formatter as minor tick formatter
    """
    # Create ax unless specfied
    if ax is None:
        ax = plt.gca()
    # Set formatter
    set_axis_tick_format_percent(ax.yaxis, fullscale, set_major=set_major, set_minor=set_minor)

def set_xaxis_tick_format_percent(fullscale=1.0, ax=None, set_major=True, set_minor=True):
    """
    Set the number of decimals and the unit for formatting the X axis.
    
    :param fullscale: What value is considered to be 100%. Use 100.0 for values that are already scaled as percent.
    :param Axis ax: You can specify an ax object here. If None, plt.gca() is used.
    :param bool set_major: If True, set the formatter as major tick formatter
    :param bool set_minor: If True, set the formatter as minor tick formatter
    """
    # Create ax unless specfied
    if ax is None:
        ax = plt.gca()
    # Set formatter
    set_axis_tick_format_datetime(ax.xaxis, fullscale, set_major=set_major, set_minor=set_minor)



#!/usr/bin/env python3
from .Figure import figure

__all__ = ["huge", "size", "medium", "medium_square", "large", "wide", "huge_wide"]

def size(w=15, h=8, fig=None):
    """
    Set the figure to the given size in inches
    """
    figure(fig).set_size_inches(w, h)

def huge(fig=None):
    """
    Make the figure huge: 20x10 inches
    """
    size(20, 10, fig)

def medium(fig=None):
    """
    Make the figure medium: 10x5 inches
    """
    size(10, 5, fig)

def medium_square(fig=None):
    """
    Make the figure a medium square: 7x7 inches
    """
    size(7, 7, fig)

def large(fig=None):
    """
    Make the figure large: 15x8 inches
    """
    size(15, 8, fig)

def wide(fig=None):
    """
    Make the figure large & wide: 15x5 inches
    """
    size(15, 5, fig)

def huge_wide(fig=None):
    """
    Make the figure large & wide: 20x7 inches
    """
    size(20, 7, fig)



#!/usr/bin/env python3
from matplotlib import pyplot as plt
import numpy as np
import matplotlib.ticker as mtick

def set_num_xticks(nticks, ax=None):
    """
    For the X axis, set a constant number of ticks, computing the
    tick spacing in <nticks> equidistant steps from start to stop.
    """
    if ax is None:
        ax = plt.gca()
    ax.xaxis.set_major_locator(mtick.LinearLocator(nticks))

def set_num_yticks(nticks, ax=None):
    """
    For the Y axis, set a constant number of ticks, computing the
    tick spacing in <nticks> equidistant steps from start to stop.
    """
    if ax is None:
        ax = plt.gca()
    ax.yaxis.set_major_locator(mtick.LinearLocator(nticks))



#!/usr/bin/env python3
from matplotlib import pyplot as plt

__all__ = ["figure"]

def figure(fig=None):
    """
    Return either fig or plt.gcf() if fig is None.
    """
    return plt.gcf() if fig is None else fig



#!/usr/bin/env python3
import openpyxl.utils.cell
from decimal import Decimal
import functools

def text_length(text):
    """
    Get the effective text length in characters, taking into account newlines
    """
    if not text:
        return 0
    lines = text.split("\n")
    return max(len(line) for line in lines)

def _to_str_for_length(v, decimals=3):
    """
    Like str() but rounds decimals to predefined length
    """
    if isinstance(v, float):
        # Round to [decimal] places
        return str(Decimal(v).quantize(Decimal('1.' + '0' * decimals)).normalize())
    else:
        return str(v)

def auto_adjust_xlsx_column_width(df, writer, sheet_name, margin=3, length_factor=1.0, decimals=3, index=True):
    """
    Auto adjust column width to fit content in a XLSX exported from a pandas DataFrame.

    How to use:
    ```
    with pd.ExcelWriter(filename) as writer:
        df.to_excel(writer, sheet_name="MySheet")
        auto_adjust_column_width_index(df, writer, sheet_name="MySheet", margin=3)
    ```

    :param DataFrame df: The DataFrame used to export the XLSX.
    :param pd.ExcelWriter writer: The pandas exporter with engine="xlsxwriter"
    :param str sheet_name: The name of the sheet
    :param int margin: How many extra space (beyond the maximum size of the string)
    :param int length_factor: The factor to apply to the character length to obtain the column width.
    :param int decimals: The number of decimal places to assume for floats: Should be the same as the number of decimals displayed in the XLSX
    :param bool index: Whether the DataFrame's index is inserted as a separate column (if index=False in df.to_xlsx() set index=False here!)
    """
    writer_type = type(writer.book).__module__ # e.g. 'xlsxwriter.workbook' or 'openpyxl.workbook.workbook'
    is_openpyxl = writer_type.startswith("openpyxl")
    is_xlsxwriter = writer_type.startswith("xlsxwriter")
    _to_str = functools.partial(_to_str_for_length, decimals=decimals)
    """
    Like str() but rounds decimals to predefined length
    """
    if not is_openpyxl and not is_xlsxwriter:
        raise ValueError("Only openpyxl and xlsxwriter are supported as backends, not " + writer_type)
    sheet = writer.sheets[sheet_name]
    # Compute & set column width for each column
    for column_name in df.columns:
        # Convert the value of the columns to string and select the 
        column_length =  max(df[column_name].apply(_to_str).map(text_length).max(), text_length(column_name))
        # Get index of column in XLSX
        # Column index is +1 if we also export the index column
        col_idx = df.columns.get_loc(column_name)
        if index:
            col_idx += 1
        # Set width of column to (column_length + margin)
        if is_openpyxl:
            sheet.column_dimensions[openpyxl.utils.cell.get_column_letter(col_idx + 1)].width = column_length * length_factor + margin
        else: # is_xlsxwriter
            sheet.set_column(col_idx, col_idx, column_length * length_factor + margin)
    # Compute column width of index column (if enabled)
    if index: # If the index column is being exported
        index_length =  max(df.index.map(_to_str).map(text_length).max(), text_length(df.index.name))
        if is_openpyxl:
            sheet.column_dimensions["A"].width = index_length * length_factor + margin
        else: # is_xlsxwriter
            sheet.set_column(0, 0, index_length * length_factor + margin)
